"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dynamodb_expressions_1 = require("@awslabs-community-fork/dynamodb-expressions");
var conditions_1 = require("./conditions");
describe('Condition Expression builder', function () {
    it('should build a valid simple comparator expression', function () {
        var expression = {
            firstname: { '=': 'Fred' },
        };
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('#attr0 = :val1');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fred');
    });
    it('should build a valid comparator expression for nested property', function () {
        var expression = {
            child: {
                name: { '=': 'Child' },
            },
        };
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('#attr0.#attr1 = :val2');
        expect(attributes.names['#attr0']).toEqual('child');
        expect(attributes.names['#attr1']).toEqual('name');
        expect(attributes.values[':val2'].S).toEqual('Child');
    });
    it('should build a valid function expression', function () {
        var expression = {
            firstname: conditions_1.beginsWith('Fre'),
        };
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('begins_with (#attr0, :val1)');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fre');
    });
    it('should build a valid size expression', function () {
        var expression = {
            firstname: conditions_1.size({ '>': 3 }),
        };
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('size (#attr0) > :val1');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].N).toEqual('3');
    });
    it('should build a valid attribute_exists expression', function () {
        var expression = {
            firstname: conditions_1.attributeExists(),
        };
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('attribute_exists (#attr0)');
        expect(attributes.names['#attr0']).toEqual('firstname');
    });
    it('should build a valid combined expression', function () {
        var expression = {
            firstname: conditions_1.beginsWith('Fre'),
            surname: conditions_1.contains('joe'),
        };
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('(begins_with (#attr0, :val1)) AND (contains (#attr2, :val3))');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fre');
        expect(attributes.names['#attr2']).toEqual('surname');
        expect(attributes.values[':val3'].S).toEqual('joe');
    });
    it('should build a valid combined expression with explicit AND operator', function () {
        var expression = conditions_1.AND({
            firstname: conditions_1.beginsWith('Fre'),
            surname: conditions_1.contains('joe'),
        });
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('(begins_with (#attr0, :val1)) AND (contains (#attr2, :val3))');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fre');
        expect(attributes.names['#attr2']).toEqual('surname');
        expect(attributes.values[':val3'].S).toEqual('joe');
    });
    it('should build a valid combined expression with explicit OR operator', function () {
        var expression = conditions_1.OR({
            firstname: conditions_1.beginsWith('Fre'),
            surname: conditions_1.contains('joe'),
            child: {
                name: conditions_1.isIn(['Mark']),
            },
        });
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('((begins_with (#attr0, :val1)) OR (contains (#attr2, :val3))) OR (#attr4.#attr5 in (:val6))');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fre');
        expect(attributes.names['#attr2']).toEqual('surname');
        expect(attributes.values[':val3'].S).toEqual('joe');
        expect(attributes.names['#attr4']).toEqual('child');
        expect(attributes.names['#attr5']).toEqual('name');
        expect(attributes.values[':val6'].S).toEqual('Mark');
    });
    it('should build a valid combined expression with explicit NOT operator', function () {
        var expression = conditions_1.NOT({
            firstname: conditions_1.beginsWith('Fre'),
            surname: conditions_1.contains('joe'),
        });
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('NOT ((begins_with (#attr0, :val1)) AND (contains (#attr2, :val3)))');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fre');
        expect(attributes.names['#attr2']).toEqual('surname');
        expect(attributes.values[':val3'].S).toEqual('joe');
    });
    it('should build a valid combined expression with explicit NOT and OR operator', function () {
        var expression = conditions_1.NOT(conditions_1.OR({
            firstname: conditions_1.beginsWith('Fre'),
            surname: conditions_1.contains('joe'),
        }));
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('NOT ((begins_with (#attr0, :val1)) OR (contains (#attr2, :val3)))');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fre');
        expect(attributes.names['#attr2']).toEqual('surname');
        expect(attributes.values[':val3'].S).toEqual('joe');
    });
    it('should build a valid combined expression with explicit NOT and OR operator and array of expressions', function () {
        var expression = conditions_1.NOT(conditions_1.OR({
            firstname: conditions_1.beginsWith('Fre'),
            surname: conditions_1.contains('joe'),
        }, conditions_1.NOT({
            age: { '<=': 18 },
        })));
        var attributes = new dynamodb_expressions_1.ExpressionAttributes();
        var expressionString = conditions_1.serializeConditionExpression(expression, attributes);
        expect(expressionString).toEqual('NOT ((begins_with (#attr0, :val1)) OR (contains (#attr2, :val3)) OR NOT (#attr4 <= :val5))');
        expect(attributes.names['#attr0']).toEqual('firstname');
        expect(attributes.values[':val1'].S).toEqual('Fre');
        expect(attributes.names['#attr2']).toEqual('surname');
        expect(attributes.values[':val3'].S).toEqual('joe');
        expect(attributes.names['#attr4']).toEqual('age');
        expect(attributes.values[':val5'].N).toEqual('18');
    });
});
