import { ConditionExpression } from './conditions';
import { DynamoMarshallerFor } from './marshalling';
import { Queryable } from './queryable';
import { DynamodbTableConfig, IndexDefinition, IndexDefinitions, TableDefinition } from './table-builder';
import { DynamoObject, DynamoPrimitive, DynamoValueKeys, NestedKeyOf, NestedPick, RecursivePartial, RequireAtLeastOne } from './types';
declare type UpdateReturnValue = 'NONE' | 'ALL_OLD' | 'UPDATED_OLD' | 'ALL_NEW' | 'UPDATED_NEW';
declare type SetOpts<A extends DynamoObject, RV extends UpdateReturnValue> = {
    readonly returnValue?: RV;
    readonly conditionExpression?: ConditionExpression<A>;
};
declare type PutOpts<A extends DynamoObject> = {
    readonly conditionExpression?: ConditionExpression<A>;
};
declare type PartSetResponse<A extends DynamoObject, RV extends UpdateReturnValue> = RV extends 'ALL_NEW' ? A : RV extends 'ALL_OLD' ? A : RV extends 'NONE' ? void : Partial<A>;
declare type SetResponse<A extends DynamoObject, RV extends UpdateReturnValue> = PartSetResponse<A, RV>;
declare type TransactWriteItem<I, CE> = {
    readonly item: I;
    readonly conditionExpression?: ConditionExpression<CE>;
};
export declare type Table<A extends DynamoObject, HK extends string, RK extends string, Ixs extends IndexDefinitions> = {
    readonly get: <AA extends A, Keys extends NestedKeyOf<AA> = NestedKeyOf<AA>>(hk: Pick<A, HK | RK>, opts?: {
        readonly consistentRead?: boolean;
        readonly marshaller?: DynamoMarshallerFor<A>;
        readonly keys?: readonly Keys[];
    }) => Promise<NestedPick<AA, Keys> | null>;
    readonly batchGet: <Keys extends NestedKeyOf<A> = NestedKeyOf<A>>(hks: readonly Pick<A, HK | RK>[], opts?: {
        readonly consistentRead?: boolean;
        readonly keys?: readonly Keys[];
    }) => Promise<readonly NestedPick<A, Keys>[]>;
    readonly transactGet: (hks: readonly Pick<A, HK | RK>[]) => Promise<readonly A[]>;
    readonly put: <AA extends A>(a: AA, opts?: PutOpts<A>) => Promise<void>;
    readonly set: <RV extends UpdateReturnValue = 'NONE'>(key: Pick<A, HK | RK>, updates: RecursivePartial<Omit<A, HK | RK>>, opts?: SetOpts<Omit<A, HK | RK>, RV>) => Promise<SetResponse<A, RV>>;
    readonly batchPut: (a: ReadonlyArray<A>) => Promise<void>;
    readonly batchDelete: (keys: ReadonlyArray<Pick<A, HK | RK>>) => Promise<void>;
    readonly transactPut: (a: ReadonlyArray<TransactWriteItem<A, A>>) => Promise<void>;
    readonly transactDelete: (keys: ReadonlyArray<TransactWriteItem<Pick<A, HK | RK>, A>>) => Promise<void>;
    readonly transactWrite: (args: RequireAtLeastOne<{
        readonly puts: ReadonlyArray<TransactWriteItem<A, A>>;
        readonly deletes: ReadonlyArray<TransactWriteItem<Pick<A, HK | RK>, A>>;
        readonly updates: ReadonlyArray<{
            readonly key: Pick<A, HK | RK>;
            readonly updates: Partial<Omit<A, HK | RK>>;
            readonly conditionExpression?: ConditionExpression<A>;
        }>;
    }>) => Promise<void>;
    readonly delete: (hk: Pick<A, HK | RK>, opts?: {
        readonly conditionExpression?: ConditionExpression<A>;
    }) => Promise<void>;
    readonly indexes: Indexes<Ixs>;
};
export declare type QueryableTable<A extends DynamoObject, HK extends string, RK extends string, Ixs extends IndexDefinitions> = Table<A, HK, RK, Ixs> & Queryable<A, HK, RK>;
export declare type TableFactoryResult<A extends DynamoObject, HK extends string, RK extends string, Ixs extends IndexDefinitions> = A[RK] extends never ? Table<A, HK, RK, Ixs> : QueryableTable<A, HK, RK, Ixs>;
export declare type TableConfig<A extends Record<string, DynamoPrimitive>, HK extends DynamoValueKeys<A> & string, RK extends DynamoValueKeys<A> & string = never> = {
    readonly hashKey: HK;
    readonly sortKey?: RK;
};
declare type Index<ID> = ID extends IndexDefinition<infer T, infer PK, infer SK> ? Queryable<T, PK, SK> : never;
declare type Indexes<ID> = ID extends IndexDefinitions ? {
    readonly [K in keyof ID]: Index<ID[K]>;
} : never;
/**
 *
 * @param table The name of the DynamoDB table
 * @param config The dynamoDb and marshalling { @link Config }
 * @param client An optional implementation of the { @link DynamoDb } client
 */
export declare const Table: <T extends DynamoObject, PartitionKey extends string & keyof T = never, SortKey extends string & keyof T = never, Ixs extends Record<string, IndexDefinition<any, any, any>> = Record<string, never>>(tableDefintion: TableDefinition<T, PartitionKey, SortKey, Ixs>, config?: DynamodbTableConfig) => TableFactoryResult<T, PartitionKey, SortKey, Ixs>;
export {};
